﻿Ext.require('App.Const.Finances');
Ext.define('App.View.Kivii.Finances.Settlement.List', {
    extend: 'System.Control.EntityList',
    entityType: 'Kivii.Finances.Entities.Settlement',
    xtype: 'App.View.Kivii.Finances.Settlement.List',
    dataSource: '/Restful/Kivii.Finances.Entities.Settlement/QueryEx.json',
    title: '结算管理',
    iconCls: 'x-fa fa-newspaper-o',
    filter: {
        IsPayed: false
    },
    viewModel: {
        data: {
            _selection: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 400,
        groupField: 'PayerName',
        sorters: [{
            property: 'OwnerName',
            direction: 'DESC'
        }, {
            property: 'PayerName',
            direction: 'DESC'
        }, {
            property: 'OperateTime',
            direction: 'DESC'
        }]
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    features: [{
        ftype: 'groupingsummary',
        groupHeaderTpl: [
            '<div>{name:this.formatName} ({rows.length} 条)</div>',
            {
                formatName: function(name) {
                    return name;
                }
            }
        ]
    }],
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'menu',
        items: [{
                xtype: 'combo',
                queryMode: 'local',
                displayField: 'displayInfo',
                valueField: 'isPayed',
                value: false,
                editable: false,
                filterPickList: true,
                reference: 'isPayed',
                store: {
                    fields: ['displayInfo', 'isPayed'],
                    data: [{
                        displayInfo: '已收款',
                        isPayed: true
                    }, {
                        displayInfo: '未收款',
                        isPayed: false
                    }]
                },
                listeners: {
                    select: '_onSelectIsPayed'
                }
            }, {
                xtype: 'datefield',
                fieldLabel: '时间范围  开始',
                format: 'Y-m-d',
                labelWidth: 90,
                reference: 'startTime',
                editable: false,
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanBeginOperateTime'
                    }
                },
                listeners: {
                    select: '_onSelectMonth'
                }
            },
            {
                xtype: 'datefield',
                fieldLabel: '结束',
                format: 'Y-m-d',
                labelWidth: 30,
                reference: 'endTime',
                editable: false,
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanEndOperateTime'
                    }
                },
                listeners: {
                    select: '_onSelectMonth'
                }
            }, {
                xtype: 'searchField',
                flex: 1,
                labelWidth: 60,
                queryField: 'PayerName,BizId,OperatorName',
                emptyText: '模糊搜索付款单位、所属业务或操作人',
            }, {
                xtype: 'checkbox',
                // value:true,
                labelWidth: 100,
                fieldLabel: '仅显示未有账单',
                iconAlign: 'left',
                handler: '_onCheck'
            }, {
                text: '新增结算',
                iconCls: 'x-fa fa-plus',
                handler: '_onCreateSettlement'
            }, {
                text: '生成账单',
                iconCls: 'x-fa fa-file-text-o',
                handler: '_onGenerateBill',
                bind: {
                    disabled: '{!_selection}'
                }
            // }, {
            //     text: '收款登记',
            //     iconCls: 'x-fa fa-tags',
            //     handler: '_onPaying',
            //     bind: {
            //         disabled: '{!_selection}'
            //     }
            }
        ]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: false
        },
        items: [{
            xtype: 'rownumberer'
        }, {
            dataIndex: 'BizId',
            text: '所属业务',
            width: 170,
            // sortable: true,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('合计');
            }
        }, {
            dataIndex: 'OwnerName',
            text: '所属部门',
            width: 120
        }, {
            dataIndex: 'OperateTime',
            text: '操作日期',
            width: 100,
            // sortable: true,
            renderer: Ext.util.Format.dateRenderer('Y-m-d')
        }, {
            dataIndex: 'AmountPlan',
            align: 'right',
            text: '原金额',
            width: 100,
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'Amount',
            align: 'right',
            text: '结算金额',
            width: 100,
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'AmountPayment',
            align: 'right',
            text: '已付金额',
            width: 100,
            renderer: '_renderAmount',
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'Type',
            text: '类型',
            width: 90,
            renderer: '_renderType'
        }, {
            dataIndex: 'OperatorName',
            text: '操作人',
            width: 90
        }, {
            xtype: 'actioncolumn',
            text: '操作',
            align: 'center',
            items: [{
                text: '查看',
                iconCls: 'x-fa fa-search',
                handler: '_onReadSettlement'
            }],
            width: 50
        }, {
            dataIndex: 'Summary',
            text: '摘要',
            width: 200
        }, {
            dataIndex: 'Remark',
            text: '备注',
            minWidth: 100,
            flex: 1
        }]
    },
    listeners: {
        activate: '_onActivate'
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _renderType: function(_value, _meta, _record, _rowIndex) {
            if (_value === "Contract") {
                return "合同费";
            }
            if (_value === "Report") {
                return "报告费";
            }
            if (_value === "Service") {
                return "服务费";
            }
            if (_value === "Express") {
                return "快递费";
            }
            return _value;
        },
        _renderAmount: function(_value, _meta, _record, _rowIndex) {
            if (_value === _record.data.Amount) {
                _meta.style = 'background-color:#B4EEB4';
            } else _meta.style = 'background-color:#F0E68C';
            if(_record.data.hasOwnProperty('OffsetKvid')&&_record.data.OffsetKvid!==null)_meta.style = 'background-color:#FF0000';
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },

        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _startTime = _refs.startTime.value;
            let _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.OperateTimeGreaterThanOrEqualTo;
            delete _view.filter.OperateTimeLessThan;
            _view.filter.OperateTimeGreaterThanOrEqualTo = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.OperateTimeLessThan = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        },
        _cleanBeginOperateTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OperateTimeGreaterThanOrEqualTo;
            _view.reload();
        },
        _cleanEndOperateTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OperateTimeLessThan;
            _view.reload();
        },
        _onSelectIsPayed: function(_ctrl, _record) {
            let _controller = this;
            let _view = _controller.getView();
            _view.filter.IsPayed = _record.data.isPayed;
            _view.reload();
        },
        _onCheck: function(_ctrl, _isChecked) {
            let _controller = this;
            let _view = _controller.getView();
            if (!_isChecked) delete _view.filter.IsBilled;
            else _view.filter.IsBilled = !_isChecked;
            _view.reload();
        },
        _onCreateSettlement: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _form = Ext.create('App.Control.Kivii.Finances.Settlement.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 1300,
                height: 720,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _win.show();
            _form.on('entityCreated', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        },
        _onReadSettlement: function(_ctrl, _recIndex, _cellIndex, _item, _e, _record) {
            let _controller = this;
            let _view = _controller.getView();
            let _form = Ext.create('App.Control.Kivii.Finances.Settlement.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 1200,
                height: 680,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_record.data);
            _form.on('entityDeleted', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        },
        _onGenerateBill: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _selecteds = _vm.data.selections || [];
            if (!_view.selection) return;
            if (_selecteds.length <= 0) return;
            
            let _bill={
                Metadata: {},
                Kvid: null,
                Amount: 0,
                AmountPlan: 0,
                AmountPayment: 0,
                OwnerName: KiviiContext.CurrentMember.DepartmentName,
                OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid,
                OperateTime: new Date(),
                PayeeKvid: KiviiContext.CurrentMember.OrganizationKvid,
                PayeeName: KiviiContext.CurrentMember.OrganizationName,
                OperatorName: KiviiContext.CurrentMember.FullName,
                OperatorKvid: KiviiContext.CurrentMember.Kvid,
                Currency: 'CNY',
                Details: []
            };
            for (let _i = 0; _i < _selecteds.length; _i++) {
            	_bill.PayerName=_selecteds[_i].data.PayerName;
            	_bill.PayerKvid=_selecteds[_i].data.PayerKvid;
                _bill.Details.push(_selecteds[_i].data);
            }
            let _form = Ext.create('App.Control.Kivii.Finances.Bill.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 1200,
                height: 700,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _form.setOwner(_bill);
            _win.show();
            _form.on('entityCreated', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        },
        _onPaying: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _selecteds = _vm.data.selections || [];
            if (!_view.selection) return;
            if (_selecteds.length <= 0) return;
            let _form = Ext.create('App.Control.Kivii.Finances.Settlement.Paying.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 900,
                height: 500,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _form.setOwner(_selecteds);
            _win.show();
            _form.on('entityPaying', function(_ctrl, _entity) {
            	_win.close();
                _view.reload();
            });
        }
    }
});