﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "发票查询")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceQueryEx : RestfulExecution<Invoice>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public DateTime? BeginTime { get; set; }
        public DateTime? EndTime { get; set; }

        public bool? Debited { get; set; }//借票到账未到账查询

        public override object OnExecution(IRequest req, IResponse res)
        {
            (BeginTime != null && EndTime != null && BeginTime.Value > EndTime.Value).ThrowIfTrue("开始日期不能大于结束时间!");
            var conn = KiviiContext.GetOpenedDbConnection<Invoice>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Invoice>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.RootKvid == o.Kvid);
            //sqlExpress.Where(o => o.RootKvid == o.Kvid && o.OffsetKvid == Guid.Empty && o.Category == InvoiceApplyType.Debit.ToString());
            if (BeginTime != null) sqlExpress.And(o => o.OperateTime >= BeginTime.Value);
            if (EndTime != null) sqlExpress.And(o => o.OperateTime < EndTime.Value);
            if (Debited != null)
            {
                if (Debited.Value) sqlExpress.And(o => o.AmountPayment == o.Amount);
                else sqlExpress.And(o => o.AmountPayment < o.Amount);
            }
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);

            if (Debited != null)
            {
                if (!Debited.Value)
                {
                    var queryAccount = conn.From<Account>();
                    queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
                    queryAccount.Select(o => o.Kvid);

                    var queryPayment = conn.From<Payment>();
                    queryPayment.Where(o => o.RootKvid == o.Kvid && o.OffsetKvid == Guid.Empty && o.AmountInvoice < o.Amount && (Sql.In(o.PayerAccountKvid, queryAccount)));
                    queryPayment.Select(o => o.PayerName);
                    var paymentNames = conn.Column<string>(queryPayment).Distinct().ToList();
                    foreach (var item in rtns.Results)
                    {
                        if (paymentNames.Exists(o => o == item.PayerName))
                        {
                            item.Type = "Debit";
                        }
                    }
                    rtns.Results = rtns.Results.OrderBy(o => o.Type).ToList();
                }
            }
            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceQuery : RestfulQuery<Invoice>
    { }

    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceRead : RestfulRead<Invoice>
    { }
}
